#include "ConfigWindow.h"

//Config window constructor
//needs qapplication to access desktop
ConfigWindow::ConfigWindow(QWidget *parent,QApplication *qapp) : QWidget(parent)
{

     imageSaver = new SimpleImageSaver(QString("./capture/pic"));

     int port = QInputDialog::getInteger(this,"QInputDialog::getInteger()",
                                        "Port:", 3000, 0, 65000, 1,NULL);

     tcpserver = new TcpServerImagehandler(port);

     selectionWindow = new SelectionWindow();
     imageRecorder = new ImageRecorder();
     GLrenderer = new GLStreamRenderer(this);
     textureSlider = new QSlider(Qt::Horizontal,this);
     textureSlider->setMinimum(0);
     textureSlider->setMaximum(0);

    //LAYOUT
     //setFixedSize(200, 200);
     QVBoxLayout *mainLayout = new QVBoxLayout;
     selectButton = new QPushButton(tr("Select area"), this);
     //selectButton->setGeometry(62, 40, 75, 30);
     selectButton->setFont(QFont("Times", 18, QFont::Bold));
     start = new QPushButton(tr("Start/Stop"), this);
     //start->setGeometry(62, 80, 75, 30);
     start->setFont(QFont("Times", 18, QFont::Bold));

     selectionLabel = new QLabel("Imagehandler:");
     selectTcpServer = new QRadioButton("Tcp Server",this);
     selectImageSaver = new QRadioButton("Image saver",this);

     fpsLabel = new QLabel("Target fps:");
     fpsBox = new QSpinBox();
     fpsBox->setRange(1,60);
     fpsBox->setValue(20);

     mainLayout->addWidget(GLrenderer,100);
     mainLayout->addWidget(textureSlider);
     mainLayout->addWidget(selectButton);
     mainLayout->addWidget(selectionLabel);
     mainLayout->addWidget(selectTcpServer);
     mainLayout->addWidget(selectImageSaver);
     mainLayout->addWidget(fpsLabel);
     mainLayout->addWidget(fpsBox);
     mainLayout->addWidget(start);
     setLayout(mainLayout);

     connect(selectButton, SIGNAL(clicked()), this, SLOT(SLOT_startSelection()));
     connect(start, SIGNAL(clicked()), this, SLOT(SLOT_startStopCapture()));
     connect(this, SIGNAL(SIG_startCapture(const ConfigStructure *)), imageRecorder, SLOT(SLOT_startCapture(const ConfigStructure *)));
     connect(this, SIGNAL(SIG_stopCapture()), imageRecorder, SLOT(SLOT_stopCapture()));

     connect(textureSlider, SIGNAL(valueChanged(int)), GLrenderer, SLOT(selectTexture(int)));

     config.observer = GLrenderer;
     config.imageHandler = imageSaver;
     config.captureRectangle = selectionWindow->getSelectionRectangle();
     config.theDesktop = qMainApp->desktop();
     config.fps = 30;

     qMainApp = qapp;
     startedCapturing = false;
}

ConfigWindow::~ConfigWindow()
{
    //dtor
}

void ConfigWindow::SLOT_startSelection()
{
    //Copy screen pixmap
    selectionWindow->setSelectionImage(QPixmap::grabWindow( (qMainApp->desktop())->screen()->winId()));
    selectionWindow->showFullScreen();
}

void ConfigWindow::SLOT_startStopCapture()
{
    if(!startedCapturing)
    {
        if(selectTcpServer->isChecked()) config.imageHandler = tcpserver;
        else config.imageHandler = imageSaver;

        config.fps=fpsBox->value();

        emit SIG_startCapture(&config);
        startedCapturing = true;
    }
    else
    {
        emit SIG_stopCapture();
        startedCapturing = false;
        textureSlider->setMaximum((imageRecorder->capturedImages)-1);
    }
    selectButton->setDisabled(startedCapturing);
    selectTcpServer->setDisabled(startedCapturing);
    fpsBox->setDisabled(startedCapturing);
    selectImageSaver->setDisabled(startedCapturing);
    textureSlider->setDisabled(startedCapturing);
}

void ConfigWindow::SLOT_captureEnded(int)
{

}
