#include "SelectionWindow.h"

SelectionWindow::SelectionWindow(QWidget *parent) : QWidget(parent)
{
    setAutoFillBackground(true);
    selectionRectangle = QRect(0,0,640,480);

     xSize = new QSpinBox(this);
     xSize->setRange(1,100000);
     xSize->setValue(640);
     ySize = new QSpinBox(this);
     ySize->setRange(1,100000);
     ySize->setValue(480);

     close = new QPushButton(tr("Close"), this);
     close->setFont(QFont("Times", 18, QFont::Bold));


     connect(xSize,SIGNAL(valueChanged(int)),this,SLOT(SLOT_updateRecSize()));
     connect(ySize,SIGNAL(valueChanged(int)),this,SLOT(SLOT_updateRecSize()));
     connect(close, SIGNAL(clicked()), this, SLOT(close()));
     isMoving = false;
}

SelectionWindow::~SelectionWindow()
{
    //dtor
}

void SelectionWindow::setSelectionImage(const QPixmap &pmap)
{
    currentPixMap = pmap;
    //setFixedSize(200,200);
    if(P_DEBUG) {std::cout << currentPixMap.width() << " " << currentPixMap.height() << std::endl;}
}

void SelectionWindow::mouseMoveEvent(QMouseEvent* event)
{

    #if 0
            std::cout << "MOUSE " << event->x() << " " << event->y() << std::endl;
                 std::cout << "RECT " << selectionRectangle.top() << " " << selectionRectangle.bottom() << std::endl;
                 std::cout << "     " << selectionRectangle.left() << " " << selectionRectangle.right() << std::endl;
                 #endif

    //If truly inside rectangle
    //add changing cursor later
    if( (selectionRectangle.contains(event->pos(),false)) || (isMoving))
    {
        QPoint mdif = (event->pos()-oldMousePoint);
        //Check if close to any border

        if( (event->y()-selectionRectangle.top()) < 20) selectionRectangle.setTop(selectionRectangle.top() + mdif.y());
        else if( (selectionRectangle.bottom()- event->y()) < 20) selectionRectangle.setBottom(selectionRectangle.bottom() + mdif.y());
        else if(selectionRectangle.right()-event->x() < 20) selectionRectangle.setRight(selectionRectangle.right() + mdif.x());
        else if(event->x()-selectionRectangle.left() < 20) selectionRectangle.setLeft(selectionRectangle.left() + mdif.x());
        else selectionRectangle.moveCenter(selectionRectangle.center()+mdif);

        update();
        isMoving = true;
    }

    oldMousePoint = event->pos();
}

void SelectionWindow::mouseReleaseEvent(QMouseEvent *event)
{
    if(P_DEBUG) {std::cout << "MOUSE PRESSED" << std::endl;}
    isMoving = false;
    connect(xSize,SIGNAL(valueChanged(int)),this,SLOT(SLOT_updateRecSize()));
    connect(ySize,SIGNAL(valueChanged(int)),this,SLOT(SLOT_updateRecSize()));
}

void SelectionWindow::mousePressEvent(QMouseEvent *event)
{
    if(P_DEBUG) {std::cout << "MOUSE RELEASED" << std::endl;}
    oldMousePoint = event->pos();
    disconnect(xSize,SIGNAL(valueChanged(int)),this,SLOT(SLOT_updateRecSize()));
    disconnect(ySize,SIGNAL(valueChanged(int)),this,SLOT(SLOT_updateRecSize()));
}

void SelectionWindow::paintEvent(QPaintEvent * event)
{
    QPainter painter(this);
    painter.setOpacity(0.3);
    painter.drawPixmap(this->frameGeometry(),currentPixMap);
    painter.setPen(Qt::black);
    painter.setBrush(Qt::Dense7Pattern);
    painter.setOpacity(1.0);
    painter.drawRect(selectionRectangle);
    close->setGeometry(selectionRectangle.x(), selectionRectangle.y(), 75, 30);
    xSize->setGeometry(selectionRectangle.x()+100, selectionRectangle.y(), 75, 30);
    xSize->setValue(selectionRectangle.width());
    ySize->setGeometry(selectionRectangle.x()+100, selectionRectangle.y()+30, 75, 30);
    ySize->setValue(selectionRectangle.height());
}

QRect* SelectionWindow::getSelectionRectangle()
{
    return &selectionRectangle;
}

void SelectionWindow::SLOT_updateRecSize()
{
    selectionRectangle.setHeight(ySize->value());
    selectionRectangle.setWidth(xSize->value());
    update();
}
